"""### Contains constants used to configure the installer."""

# Repository names
XMIPP = 'xmipp'
XMIPP_CORE = 'xmippCore'
XMIPP_VIZ = 'xmippViz'
XMIPP_SOURCES = [XMIPP_CORE, XMIPP_VIZ]

# Branch names
MAIN_BRANCHNAME = 'main'

# Context
VERSIONS_CONTEXT_KEY = "versions"

# Others
TAIL_LOG_NCHARS = 300
