"""### URLs."""

DOCUMENTATION_URL = 'https://i2pc.github.io/docs/'
CMAKE_INSTALL_DOCS_URL = 'https://i2pc.github.io/docs/Installation/Requirements/index.html#cmake'
API_URL = 'https://xmipp.i2pc.es/api/attempts/'
MODELS_URL= "http://scipion.cnb.csic.es/downloads/scipion/software/em"
SCIPION_TESTS_URL = "http://scipion.cnb.csic.es/downloads/scipion/data/tests"
I2PC_REPOSITORY_URL = "https://github.com/i2pc/"
DLTK_DOCS_URL = "https://i2pc.github.io/docs/Installation/Optionals/index.html#deeplearningtoolkit"
