"""### Containis all common constants needed for the argument parsing part of Xmipp's installation."""

from xmipp3_installer.installer.constants import paths

XMIPP_PROGRAM_NAME = "xmipp"

# Other variables
COMMON_USAGE_HELP_MESSAGE = 'Run \"./xmipp -h\" for usage help.'
DEFAULT_MODELS_DIR = paths.INSTALL_PATH
